#!/usr/bin/env python3
import re
import sys
import requests
import datetime
import calendar
import argparse
import json

from collections import OrderedDict

from .logs.dayutils  import dayutils
from .utils import NameParser

from . import HRauth

class HRget(object):

    def __init__(self, HRauth, verbose=False):
        self.verbose = verbose
        self.host = HRauth.host()

        # set URLs
        self.login_url    = 'https://' + self.host + '/HRPortal/servlet/cp_login'
        self.sheet_url    = 'https://' + self.host + '/HR-WorkFlow/servlet/hfpr_bcapcarte'
        self.post_url     = 'https://' + self.host + '/HR-WorkFlow/servlet/SQLDataProviderServer'
        self.portal_url   = 'https://' + self.host + '/HRPortal/servlet/SQLDataProviderServer'
        self.presence_url = 'https://' + self.host + '/HRPortal/servlet/Report?ReportName=AAA_ElencoPresenti&m_cWv=Rows%3D0%0A0%5Cu0023m_cMode%3Dhyperlink%0A0%5Cu0023outputFormat%3DCSV%0A0%5Cu0023pageFormat%3DA4%0A0%5Cu0023rotation%3DLANDSCAPE%0A0%5Cu0023marginTop%3D7%0A0%5Cu0023marginBottom%3D7%0A0%5Cu0023marginLeft%3D7%0A0%5Cu0023hideOptionPanel%3DT%0A0%5Cu0023showAfterCreate%3DTrue%0A0%5Cu0023mode%3DDOWNLOAD%0A0%5Cu0023ANQUERYFILTER%3D1%0A0%5Cu0023pRAPPORTO%3D%0A0%5Cu0023pFILIALE%3D%0A0%5Cu0023pUFFICIO%3D%0A0%5Cu0023m_cParameterSequence%3Dm_cMode%2CoutputFormat%2CpageFormat%2Crotation%2CmarginTop%2CmarginBottom%2CmarginLeft%2Cmode%2ChideOptionPanel%2CshowAfterCreate%2CANQUERYFILTER%2CpRAPPORTO%2CpFILIALE%2CpUFFICIO%0A'

        # set employee
        self.username = HRauth.username()
        self.password = HRauth.password()
        self.idemploy = "{:0>7}".format(str(HRauth.idemploy()))
        #print(self.idemploy)

        if self.verbose > 1:
           print (">>>USERNAME>>>", self.username)
           print (">>>IDEMPLOY>>>", self.idemploy)
           #print ("[{}]@{}".format(__class__.__name__, sys._getframe().f_code.co_name))

        self.session = requests.Session()
        self.cookies = None 

        self.login()

    def login(self):
        auth = {'m_cUserName' : self.username, 'm_cPassword' : self.password, 'm_cAction' : 'login'}
        r = self.session.post(self.login_url, params=auth, allow_redirects=False)
        self.cookies = r.cookies

        if self.verbose > 1:
           print (">>>CODE>>>", r.status_code)
           print (">>>HISTORY>>>", r.history)
           print (">>>HEADERS>>>", r.headers)
           print (">>>COOKIES>>>", r.cookies)
           print (">>>LOCATION>>>", r.headers['location'])
           #print (">>>PAGE>>>", r.text)

        try:
           if 'jsp/home.jsp' in r.headers['location']:
              pass
        except:
           print("\n[HRget] *** ERROR *** on HR authentication!\n")
           sys.exit(1)

    def get_range(self, day_range):
       data = []
       # loop over months of day_range
       for y, m in day_range.months():

          # get month data from HR
          full_month = self.get(y, m)

          # get first and last days for month
          first, last = dayutils.month_bounds( datetime.date(y, m, 1) )
          # to avoid future day: if same month of today's month set last day to today
          if dayutils.is_same_month( datetime.datetime(y, m, 1), datetime.datetime.today() ):
             last = datetime.datetime.today().date()
          # list of month days within day_range
          l = [ i.day for i in dayutils.day_range(first, last) if i in day_range ]
          # extract days within day_range from data of all month
          old_data = full_month['Data']
          data += [ old_data[i-1] for i in l ]

          # save fields
          fields = full_month['Fields']

       if data:
         jret = {'Fields' : fields, 'Data' : data}
         return jret


    def _check_data(self, d, year, month, day):

        if len(d) < 1:
            print("[HRget] ***ERROR***")
            print("Data not found in date {}-{}".format(year, month) )
            sys.exit(-1)


    def get(self, year  = datetime.datetime.today().year,
                  month = datetime.datetime.today().month,
                  day   = None):

        str_month = "{:0>2}".format(month)
        str_year  = "{:d}".format(year)

        num_days_in_month = calendar.monthrange(year, month)[1]

        last_day = num_days_in_month
        if dayutils.is_same_month( datetime.datetime(year, month, 1), datetime.datetime.today() ):
           last_day = datetime.datetime.today().day

        if self.verbose > 1:
            print (">>>NUMDAYMONTH>>>", num_days_in_month)
            print (">>>LASTDAY>>>", last_day)

        # COOKIES {{{
        cookies = self.cookies
        # }}}
        # HEADERS {{{
        headers = {
                   'Pragma': 'no-cache',
                   'Cache-Control': 'no-cache'
                  }
        # }}}
        # PARAMS  {{{
        params = {
                  'rows' : 300,
                  'startrow' : '0',
                  'count' : 'false',
                  'cmdhash':'89ff07d888efecba391f40eac7d04e9a',
                  'sqlcmd' : 'rows:hfpr_fcartellino3',
                  'IDCOMPANY':'000001',
                  'IDEMPLOY': self.idemploy,
                  'Anno': str_year,
                  'Mese': str_month,
                  'Visualiz':'N',
                  'LaFlexi':'N',
                  'LaFlexiProg':'N',
                  'VisDescr':'',
                  'DADRATIM':'',
                  'DATTMI':'',
                  'HHMM':'',
                  'ImportVoci':'S',
                  'p_NOEDIT_1':'',
                  'p_NOEDIT_2':'',
                  'p_NOEDIT_3':'',
                  'p_NOEDIT_4':'',
                  'p_ADMIN':'',
                  'gCarOri':'N',
                  'gVismmAtt':'',
                  'TACART':'N',
                  'vIDPLANTB':'99999',
                  'vIDPRESGRP':'',
                  'NUMERORIGHE':'2',
                  'StringaCol':'01S&&&02S&&&03S&&&04S&&&05S&&&06S&&&07S&&&08S&&&09N&&&10N&&&11N&&&12N&&&13N&&&14N&&&15N&&&16N&&&17N&&&',
                  'SchedeLavoro':'N',
                  'pFILESITO':'',
                  'pFILGIUS':'0',
                  'TOTHORD':'S',
                  'TOTHECC':'',
                  'TOTFLEX':'',
                  'ABILITANOTESPESE':'N',
                  'TIPONOTESPESE':'1',
                  'ABILITATIMESHEET':'N'
                 }
        # }}}

        p = self.session.post(self.sheet_url, cookies=cookies)

        p = self.session.post(self.post_url, headers=headers, cookies=cookies, params=params)


        d = p.json()['Data'][:last_day]
        f = p.json()['Fields']

        self._check_data(d, year, month, day)

        if day:
            d = d[day-1]
        
        if self.verbose > 1:
            print ('>>>FIELDS>>>', f)
            print ('>>>DATA>>>', d)

            #for i, item in enumerate(f):
            #    print( item, ' = ', d[i])

        json = {'Fields' : f, 'Data' : d}

        return json


    def totalizators(self,
                     year  = datetime.date.today().year,
                     month = datetime.date.today().month):

        date = datetime.date(year, month, 1) + datetime.timedelta(days=calendar.monthrange(year, month)[1])

        # COOKIES {{{
        cookies = self.cookies
        # }}}
        # HEADERS {{{
        headers = {
                   'Pragma': 'no-cache',
                   'Cache-Control': 'no-cache'
                  }
        # }}}
        # PARAMS {{{
        params = {
                  'rows' : 100,
                  'startrow' : '0',
                  'count' : 'false',
                  'cmdhash':'2e2926382311a0b72e72cd1e1cdc4ccc',
                  'sqlcmd' : 'rows:hfpr_fgadgetconta',
                  'pIDEMPLOY': self.idemploy,
                  'pIDCOMPANY':'000001',
                  'pDATA':date,
                  'pADMIN':'',
                 }
        # }}}

        p = self.session.post(self.sheet_url, cookies=cookies)

        p = self.session.post(self.post_url, headers=headers, cookies=cookies, params=params)

        d = p.json()['Data'][:-1]
        f = p.json()['Fields']

        json = {'Fields' : f, 'Data' : d}

        return json


    def phone(self, names = [], phones = []):

        re_names = [ re.compile(i.upper()) for i in names ]
        re_phones = [ re.compile(i.upper()) for i in phones ]

        fields_name = ['ANSURNAM']
        fields_phone = ['ANTELEF', 'ANMOBILTEL']

        fields_to_return = ['ANSURNAM', 'ANEMAIL', 'ANTELEF', 'ANMOBILTEL']
        data_to_return   = []


        # COOKIES {{{
        cookies = self.cookies
        # }}}
        # HEADERS {{{
        headers = {
                   'Pragma': 'no-cache',
                   'Cache-Control': 'no-cache'
                  }
        # }}}
        # PARAMS  {{{
        params = {
                  'rows'      : '2000',
                  'startrow'  : '0',
                  'count'     : 'true',
                  'cmdhash'   : 'b55cc94f7a3a372690c14949975ac422',
                  'sqlcmd'    : 'q_rubrica',
                  'pANSURNAM' : '',
                 }
        # }}}

        p = self.session.post(self.portal_url, headers=headers, cookies=cookies, params=params)

        try:
           list_phone = p.json()['Data'][:-1]
        except:
           return OrderedDict()

        fields_ = p.json()['Fields']

        # convert data to ordered dict

        json_ = {}

        for worker in list_phone:
           json = {k: v for k, v in zip(fields_, worker)}
           l = []
           for k in fields_to_return:
              if json.get(k):
                 l.append(json[k])
           data_to_return.append(l)

        json_['Fields'] = fields_to_return
        json_['Data']   = data_to_return


        # filter data
        if re_names or re_phones:
           l = []
           for i in json_['Data']:
               _name  = i[0]
               _phone = " ".join( [ re.sub(r"[^0-9]+", "", j) for j in i[2:] ] )
               l.extend( [ i for r in re_names if r.search(_name) ] )
               l.extend( [ i for r in re_phones if r.search(_phone) ] )

           json_['Data'] = l

        return json_

#{{{ deprecated
    def phone_old(self, names):

        fields_to_return = ['ANSURNAM', 'ANEMAIL', 'ANTELEF', 'ANMOBILTEL']
        data_to_return   = []

        for n in names:

            name = n.upper()

            # COOKIES {{{
            cookies = self.cookies
            # }}}
            # HEADERS {{{
            headers = {
                       'Pragma': 'no-cache',
                       'Cache-Control': 'no-cache'
                      }
            # }}}
            # PARAMS  {{{
            params = {
                      'rows'      : '5',
                      'startrow'  : '0',
                      'count'     : 'true',
                      'cmdhash'   : '8a0d2d1c35e16e1e2135a25505b5dd32',
                      'sqlcmd'    : 'q_rubrica',
                      'queryfilter' :"ANSURNAM like '" + name + "%'",
                      'pANSURNAM' : '',
                     }
            # }}}

            p = self.session.post(self.portal_url, headers=headers, cookies=cookies, params=params)

            try:
               list_phone = p.json()['Data'][:-1]
            except:
               return OrderedDict()

            fields_ = p.json()['Fields']

            # convert data to ordered dict

            json_ = {}

            for worker in list_phone:
               json = {k: v for k, v in zip(fields_, worker)}
               l = []
               for k in fields_to_return:
                  if json.get(k):
                     l.append(json[k])
               data_to_return.append(l)

        json_['Fields'] = fields_to_return
        json_['Data']   = data_to_return

        return json_
#}}}


    def presence(self):

        p = self.session.get(self.presence_url)

        csv_data = p.text

        return csv_data




def add_parser(parser):

   date_parser = parser.add_argument_group('date options')

   date_parser.add_argument('-d', '--day',
                            default = datetime.datetime.today().day, type=int,
                            help='select day')

   date_parser.add_argument('-m', '--month',
                            default = datetime.datetime.today().month, type=int,
                            help='select month')

   date_parser.add_argument('-y', '--year',
                            default = datetime.datetime.today().year, type=int,
                            help='select year')


def add_parser_phone(parser):

   phone_parser = parser.add_argument_group('phone number options')

   phone_parser.add_argument('-p', '--phone',
                             dest = 'phone_name',
                             default = [],
                             nargs = '+',
                             action = NameParser,
                             metavar = "SURNAME",
                             help="get phone number")

   phone_parser.add_argument('-n', '--name-from-phone',
                             dest = 'phone_number',
                             default = [],
                             nargs = '+',
                             metavar = "PHONE",
                             help="get name from phone number")


def main ():

    parser = argparse.ArgumentParser(prog='HRget',
                                     description='',
                                     formatter_class=argparse.RawTextHelpFormatter)

    add_parser(parser)
    add_parser_phone(parser)

    parser.add_argument('-v', '--verbose',
                        action="count", default=0,
                        help="increase verbosity")

    parser.add_argument('-g', '--get',
                        action='store_true',
                        help="get day")

    parser.add_argument('--totalizators',
                        action='store_true',
                        help="get totalizators")

    parser.add_argument('--presence',
                        action='store_true',
                        help="get presence of worker")

    parser.add_argument('--dump',
                        dest = 'file_out',
                        help="dump to file")

    HRauth.add_parser(parser)

    args = parser.parse_args()


    auth = HRauth.HRauth(**vars(args))

    hr_get = HRget(auth, verbose=args.verbose)


    if args.get:

        djson = hr_get.get(year=args.year, month=args.month, day=args.day)

        if args.verbose:
            for k, v in zip(djson['Fields'], djson['Data']):
                print(k, " = ", v)

        if args.file_out:
            with open(args.file_out, 'w') as f:
                json.dump(djson, f)
            #with open(args.file_out, 'r') as f:
            #    djson = json.load(f)


    if args.totalizators:

        djson = hr_get.totalizators(year=args.year, month=args.month)

        if args.verbose:
            for d in djson['Data']:
                for k, v in zip(djson['Fields'], d):
                    print(k, " = ", v)
                print()

        if args.file_out:
            with open(args.file_out, 'w') as f:
                json.dump(djson, f)


    if args.phone_name or args.phone_number:

        djson = hr_get.phone(names = args.phone_name, phones = args.phone_number)

        print()
        for d in djson['Data']:
            for k, v in zip(djson['Fields'], d):
                print(v)
            print()

        if args.file_out:
            with open(args.file_out, 'a') as f:
                json.dump(djson, f)


    if args.presence:

        csv = hr_get.presence()

        if args.verbose:
            print(csv)

        if args.file_out:
            with open(args.file_out, 'w') as f:
                f.write(csv)



if __name__ == '__main__':
    main()




#!/usr/bin/env python3
import re
import os
import sys
import requests

import getpass
import base64

try:
   from configparser import SafeConfigParser
except:
   from ConfigParser import SafeConfigParser


class HRauth(dict):

   HRauth_config_option = 'HRauth'

   def __init__(self, **kwargs):

      dict.__init__(self, **kwargs)

      if self['remove_config_file']:
          self._remove_config_file()

      self['required'] = ['host', 'username', 'idemploy']

      # read from config_file
      self.update( self._read_config_file(kwargs['config_file']) )
      # read from args
      self.update( {k: kwargs[k] for k in self['required'] if kwargs.get(k, None)} )

      self._check_required()

      self._get_password()

      if self['save']:
          if self.login():
              self._write_config_file(self['config_file'])


   def _get_password(self):

      if self.get('password', False):
         self['password'] = self._password_decode()
      else:
         self['password'] = getpass.getpass()


   def _password_decode(self):
      """Decode password"""

      _enc  = self['password_encoding']
      _pass = self['password']

      if   _enc == 'clear':
         _pass = _pass
      elif _enc == 'base64':
         _pass = base64.b64decode(_pass)

      return _pass

   def _password_encode(self):
      """Encode password"""

      _enc  = self['password_encoding']
      _pass = self['password']


      if   _enc == 'clear':
         _pass = self['password']
      elif _enc == 'base64':
         if isinstance(_pass, bytes):
            _pass = base64.b64encode(_pass)
         else:
            _pass = base64.b64encode(_pass.encode('ascii'))

      if isinstance(_pass, bytes):
          _pass = _pass.decode()

      return _pass

   def _check_required(self):
      missing = []
      for k in self['required']:
         if not self.get(k, False):
            missing.append(k)

      if not missing:
         return
      else:
         print("[HRauth] *** ERROR *** missing required args :", ", ".join(missing))
         sys.exit(1)


   def _read_config_file(self, fname):
      """Read from config file"""

      if not os.path.isfile(fname):
         return {}

      parser = SafeConfigParser()
      parser.read(fname)

      return {k: parser.get(self.HRauth_config_option, k) for k in parser.options(self.HRauth_config_option)}


   def _write_config_file(self, fname):
       parser = SafeConfigParser()

       parser.add_section(self.HRauth_config_option)

       if self.get('host', None):
          parser.set(self.HRauth_config_option, 'host', self['host'])
       if self.get('username', None):
          parser.set(self.HRauth_config_option, 'username', self['username'])
       if self.get('idemploy', None):
          parser.set(self.HRauth_config_option, 'idemploy', str(self['idemploy']))

       if self['save_password']:
          _encoded = self._password_encode()
          parser.set(self.HRauth_config_option, 'password', _encoded)


       with open(fname, "w") as f:
          parser.write(f)

   def _remove_config_file(self):
       if os.path.isfile(self['config_file']):
         print("Removing config file :", self['config_file'])
         os.remove(self['config_file'])


   def login(self):

       auth = {'m_cUserName' : self['username'], 'm_cPassword' : self['password'], 'm_cAction' : 'login'}

       _login_url  = 'https://' + self['host'] + '/HRPortal/servlet/cp_login'

       _session = requests.Session()

       r = _session.post(_login_url, params=auth, allow_redirects=False)

       try:
          if 'jsp/home.jsp' in r.headers['location']:
             return True
       except:
          print("\n[HRauth] *** ERROR *** on HR authentication!\n")
          sys.exit(1)


   def host(self):
      return self['host']
   def username(self):
      return self['username']
   def idemploy(self):
      return self['idemploy']
   def password(self):
      return self['password']


def add_parser(parser):

   authparser = parser.add_argument_group('authentication options')

   authparser.add_argument('-u', '--username',
                           help='Username')

   authparser.add_argument('-i', '--idemploy',
                           help='ID employee')

   authparser.add_argument('-a', '--url',
                           dest='host',
                           help='HR url')

   authparser.add_argument('-c', '--config-file',
                           default = os.path.join( os.path.expanduser("~"), '.HRlo'),
                           help='Configuration file (default %(default)s)')

   authparser.add_argument('-s', '--save',
                           action='store_true',
                           help='Save HR authentication options in HRauth config file')

   authparser.add_argument('--save-password',
                           action='store_true',
                           help='Save HR authentication password in HRauth config file')

   authparser.add_argument('--password-encoding',
                           choices=['clear', 'base64'], default='base64',
                           help='Password encoding (default %(default)s)')

   authparser.add_argument('--remove-config-file',
                           action='store_true',
                           help='Remove HR config file')

   args = parser.parse_args()
   return vars(args)


def main ():

   import HRget
   import argparse

   parser = argparse.ArgumentParser(prog='',
                                    description='descriptions',
                                    formatter_class=argparse.RawTextHelpFormatter)
   add_parser(parser)

   args = parser.parse_args()

   #print(args)

   auth = HRauth(**vars(args))

   hrget = HRget.HRget(auth)

   if hrget:
      print("Successfully login!")


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
import datetime

from .logs import dayutils

from . import HRday

class HRdayList(list):

    def __init__(self, label = None):

       self.hrday = HRday.HRday()

       #self_hr_time = {}
       #self._hr_time['net'] = 0
       #for k in list(HRday.HRday.time_hash.keys()) + ['up']:
       #    self._hr_time[k] = datetime.timedelta(0)

       self.label = label


    def __str__(self):
       s = ''

       if self.label:
          s += "."*25 + "{}\n".format(self.label)

       s += "{:.<25}".format("Working days")
       s += "{}\n".format( ", ".join([ str(i.day().date()) for i in self if i.is_working()]))
       s += "{:.<25}".format("Num. Working days")
       s += "{}\n".format( self.working_days_number() )
       #s += "{:.<25}".format("Working days list")
       #s += "{}\n".format( self.working_days_list() )
       s += "{:.<25}".format( "Uptime" )
       s += "{:<10}".format( dayutils.sec2str(self.hrday['uptime'].total_seconds()) )
       s += " {} ".format( "for HR" )
       s += "{}\n".format( dayutils.sec2str(self.hrday['HR times']['up'].total_seconds()) )
       s += "{:.<25}".format( "Timenet" )
       s += "{:<10}".format( dayutils.sec2str(self.hrday.timenet()) )
       s += " {} ".format( "for HR" )
       s += "{}\n".format( dayutils.sec2str(self.hrday['HR times']['net']) )
       s += "{:.<25}".format( "Uptime mean" )
       s += "{}\n".format( dayutils.sec2str(self.uptime_mean().total_seconds()) )
       s += "{:.<25}".format( "Timenet mean" )
       s += "{}\n".format( dayutils.sec2str(self.timenet_mean()) )
       #s += "{:.<25}".format("Logs")
       #for j in self:
       #   s += "[{}] ".format(", ".join([ i.time().strftime(dayutils.fmt_time) for i in j.logs() ]))
       #s += '\n'

       s += "{:.<25}".format("Timenets")
       s += "[{}]\n".format( ", ".join([ dayutils.sec2str(d.timenet()) for d in self if d.is_working()]) )

       s += "{:.<25}".format( "Anomaly" )
       s += "{}\n".format( self.anomaly() )

       return s

    def append(self, args):
       #if not args.is_working(): return
       list.append(self, args)

       self.hrday['uptime'] += args.uptime()

       for k in list(HRday.HRday.time_hash.keys()) + ['up', 'net']:
          self.hrday['HR times'][k] += args['HR times'][k]

       if not args.anomaly() and not args.is_holiday():
          self.hrday['timenet'] += args.timenet()


    def _getattr_from_HRday(self, attr):
       return [getattr(i, attr)() for i in self]

    def uptime_list(self):
       return self._getattr_from_HRday('uptime') 

    def uptime(self):
       up = datetime.timedelta(0)
       for i in self.uptime_list():
          up += i
       return up 

    def anomaly_list(self):
       return self._getattr_from_HRday('anomaly') 

    def anomaly(self):
       return sum(self.anomaly_list())

    #def timenet_list(self):
    #   return self._getattr_from_HRday('timenet')
    #def timenet(self):
    #   return sum(self.timenet_list())

    def is_working_list(self):
       return self._getattr_from_HRday('is_working')

    def working_days_number(self):
       return sum(self.is_working_list())
    def working_days_list(self):
       return [ i.date() for i in self if i.is_working() ]

    def timenet_mean(self):
       if self.working_days_number() == 0:
          return 0
       return self.hrday.timenet() / self.working_days_number()

    def uptime_mean(self):
       if self.working_days_number() == 0:
          return datetime.timedelta(0)
       return self.uptime() / self.working_days_number()




#!/usr/bin/env python3
import re
import sys
import datetime
import argparse

from .logs.dayutils import dayutils
from .utils import NameParser

from . import HRauth
from . import HRget
from . import HRday
from . import HRdayList
from . import HRpresence
from . import HRtotalizator

from . import color
from . import config as HRconfig

class HRlo(object):

   def __init__(self, dauth, config = {}, day_range=None):
       self.auth = HRauth.HRauth(**dauth)
       self.hrget = HRget.HRget(self.auth)

       self.data = None

       self.day_range = day_range

       self.config = {}
       self.config.update( config )

   def __str__ (self):
       s = ''
       for i in self.data:
           s += str(i) + '\n'
       return s

   def __getitem__(self, key):
       if not self.data:
           raise KeyError("no data for {}".format(key))
       if not isinstance(key, datetime.datetime) and \
          not isinstance(key, datetime.date) and \
          not isinstance(key, dayutils.day_range) and \
          not isinstance(key, slice):
          print("@__getitem__ NOT datetime.datetime")
          return None
       if isinstance(key, dayutils.day_range):
          l = [ d for d in self.data if d.day().date() in key ]
          return l
       elif isinstance(key, datetime.datetime):
          l = [ d for d in self.data if d.day().date() == key.date() ]
          return l
       elif isinstance(key, datetime.date):
          l = [ d for d in self.data if d.day().date() == key ]
          return l
       elif isinstance(key, slice):
          #print( key.start.day-1, key.stop.day-1)
          return self.data[key.start.day-1:key.stop.day-1]
       else:
          return self.data[key.day-1]


   def init_data(self, day_range=None):
       if day_range:
          json = self.hrget.get_range(day_range)
       else:
          json = self.hrget.get()

       fields, HRdata = json['Fields'], json['Data']
       self.data = [ HRday.HRday(fields, day) for day in HRdata]


   def get_report_day(self, day = datetime.date.today()):
       self.init_data( dayutils.day_range(day, day) )
       return self


   def get_report_week(self, day = datetime.datetime.today()):

       start, end = dayutils.week_bounds(day)

       return self.get_report(start, end, label="Weekly report")


   def get_report_month(self, day = datetime.datetime.today()):

       start, end = dayutils.month_bounds(day)

       return self.get_report(start, end, label="Monthly report")


   def get_report(self, start, end, label = ''):

       day_range = dayutils.day_range(start, end)

       self.init_data(day_range)

       if start == end:
           _label = str(start)
       else:
           _label = "From {} To {}".format( str(start), str(end) )

       if label:
           _label = "{} : {}".format( label, _label)

       l = HRdayList.HRdayList(label=_label)
       for i in self[day_range]:
           if i.is_today() and not self.config.get('today', False): continue
           l.append(i)
       return l


   def anomalies(self):
       _anomalies = [d for d in self.data if d.anomaly()]
       if _anomalies:
           col = color.color(color.RED)( str )
           warning = "WARNING : {} anomalies found : {}".format(len(_anomalies),  [ str(d.day().date()) for d in _anomalies ] )
           print( col(warning) )
       return _anomalies

   def get_report_all(self):

       day = self.get_report_day()
       #print(day)
       week = self.get_report_week()
       #print(week)
       month = self.get_report_month()
       #print(month)

       return day, week, month

   def report(self):
       d, w, m = self.get_report_all()
       print(d)
       print(w)
       print(m)

   def get_phone(self, surname):
       return self.hrget.phone(surname)

   def get_presence(self, surname):
       csv_data = self.hrget.presence()
       presence = HRpresence.HRpresence(csv_data)
       return presence.report(surname)

   def get_totalizator(self, key=None):
       hr_tot = HRtotalizator.HRtotalizator(self.hrget.totalizators())
       if key:
           return hr_tot.get_value(key)
       else:
           return hr_tot.report()


def main():

   parser = argparse.ArgumentParser(prog='HRlo (aka accaerralo)',
                                    description='',
                                    formatter_class=argparse.RawTextHelpFormatter)

   parser.add_argument('--version', action='version',
                       version='%(prog)s ' + HRconfig.version,
                       help='print version and exit')


   parser_todo = parser.add_argument_group('report options')

   parser_todo.add_argument('-d', '--daily',
                            action='store_true',
                            help='daily report')

   parser_todo.add_argument('-w', '--weekly',
                            action='store_true',
                            help='weekly report')

   parser_todo.add_argument('-m', '--monthly',
                            action='store_true',
                            help='monthly report')

   parser_todo.add_argument('-t', '--today',
                            action='store_true',
                            help='keep today in reports')


   parser_range = parser.add_argument_group('range days options')

   def _date(s):
      for fmt in ['%Y-%m-%d', '%Y%m%d', '%d-%m-%Y', '%d/%m/%Y']:
         try:
            return datetime.datetime.strptime(s, fmt).date()
         except:
            pass
      raise ValueError("invalid date {}".format(s))

   parser_range.add_argument("--from",
                             dest = 'from_day',
                             type=_date,
                             default=None, #default=datetime.datetime(1970, 1, 1),
                             metavar="YYYY-MM-DD",
                             help="from date YYYY-MM-DD")

   parser_range.add_argument("--to",
                             dest = 'to_day',
                             type=_date,
                             #default=None,
                             default=datetime.date.today(),
                             metavar="YYYY-MM-DD",
                             help="to date YYYY-MM-DD (default %(default)s)")

   HRpresence.add_parser(parser)

   HRget.add_parser_phone(parser)

   HRtotalizator.add_parser(parser)

   dauth = HRauth.add_parser(parser)

   args = parser.parse_args()

   config = {'today' : args.today}

   #config = {}

   hr = HRlo(dauth, config)

   if args.from_day and args.to_day:
      print(hr.get_report(args.from_day, args.to_day))

   if args.daily:
      print(hr.get_report_day())

   if args.weekly:
      print(hr.get_report_week())

   if args.monthly:
      print(hr.get_report_month())

   if args.phone_name or args.phone_number:
       djson = hr.hrget.phone(names = args.phone_name, phones = args.phone_number)
       print()
       for d in djson['Data']:
           for k, v in zip(djson['Fields'], d):
               print(v)
           print()

   if args.presence:
      for name in args.presence:
          p = hr.get_presence(name)
          print(p)

   if args.totalizators or args.get_totalizator:
       print(hr.get_totalizator(args.get_totalizator))

   if not args.daily and not args.weekly and not args.monthly \
      and not args.from_day \
      and not args.phone_name and not args.phone_number and not args.presence \
      and not args.totalizators and not args.get_totalizator:
      today = hr.get_report_day()
      if today:
          print("\nToday :")
          print(today)


if __name__ == '__main__':
   main()




#!/usr/bin/env python3
from distutils.core import setup

try:
   from distutils.command.build_py import build_py_2to3 as build_py
except ImportError:
   from distutils.command.build_py import build_py

from HRlo import config

setup(

   name=config.name,
   version=config.version,
   description=config.description,
   author=config.author,
   author_email=config.author_email,
   url=config.url,
   packages=['HRlo', 'HRlo/logs', 'HRlo/logs/dayutils'],
   scripts = ['bin/accaerralo', 'bin/HRlo',
              'bin/HRday', 'bin/HRget',
              'bin/HRpresence', 'bin/HRcompany',
              'bin/HRtotalizator',
              'bin/HRutils'],
   cmdclass = {'build_py': build_py},

   classifiers=[
       'Development Status :: 3 - Alpha',
       'Programming Language :: Python :: 3',
       'Programming Language :: Python :: 3.2',
       'Programming Language :: Python :: 3.3',
       'Programming Language :: Python :: 3.4',
   ],

)
